@@ -376,24 +376,11 @@ cleanup_for_exit ()
 }
 
 void
-scm_i_init_guile (SCM_STACKITEM *base)
+scm_i_init_guile (void *base)
 {
   if (scm_initialized_p)
     return;
 
-  if (base == NULL)
-    {
-      fprintf (stderr, "cannot determine stack base!\n");
-      abort ();
-    }
-
-  if (sizeof (mpz_t) > (3 * sizeof (scm_t_bits)))
-    {
-      fprintf (stderr,
-               "GMP's mpz_t must fit into a double_cell,"
-               "but doesn't seem to here.\n");
-    }
-
   scm_storage_prehistory ();
   scm_threads_prehistory (base);  /* requires storage_prehistory */
   scm_weaks_prehistory ();        /* requires storage_prehistory */
@@ -461,8 +448,8 @@ scm_i_init_guile (SCM_STACKITEM *base)
   scm_init_numbers ();
   scm_init_options ();
   scm_init_pairs ();
-#ifdef HAVE_POSIX
   scm_init_filesys ();     /* Requires smob_prehistory */
+#ifdef HAVE_POSIX
   scm_init_posix ();
 #endif
 #ifdef HAVE_REGCOMP
