@@ -22,9 +22,13 @@
 # include <config.h>
 #endif
 
+#define SCM_BUILDING_DEPRECATED_CODE
+
 #include "libguile/_scm.h"
 
 #include "libguile/alist.h"
+#include "libguile/deprecation.h"
+#include "libguile/deprecated.h"
 #include "libguile/eval.h"
 #include "libguile/procs.h"
 #include "libguile/gsubr.h"
@@ -39,11 +43,13 @@
 
 
 SCM_GLOBAL_SYMBOL (scm_sym_system_procedure, "system-procedure");
+#if (SCM_ENABLE_DEPRECATED == 1)
 SCM_GLOBAL_SYMBOL (scm_sym_arity, "arity");
+#endif
 SCM_GLOBAL_SYMBOL (scm_sym_name, "name");
 
-static SCM props;
-static scm_i_pthread_mutex_t props_lock = SCM_I_PTHREAD_MUTEX_INITIALIZER;
+static SCM overrides;
+static scm_i_pthread_mutex_t overrides_lock = SCM_I_PTHREAD_MUTEX_INITIALIZER;
 
 int
 scm_i_procedure_arity (SCM proc, int *req, int *opt, int *rest)
@@ -71,9 +77,30 @@ scm_i_procedure_arity (SCM proc, int *req, int *opt, int *rest)
   return scm_i_program_arity (proc, req, opt, rest);
 }
 
-/* FIXME: instead of the weak hash, perhaps for some kinds of procedures, use
-   other means; for example subrs have their own property slot, which is unused
-   at present. */
+SCM_DEFINE (scm_procedure_minimum_arity, "procedure-minimum-arity", 1, 0, 0, 
+           (SCM proc),
+	    "Return the \"minimum arity\" of a procedure.\n\n"
+            "If the procedure has only one arity, that arity is returned\n"
+            "as a list of three values: the number of required arguments,\n"
+            "the number of optional arguments, and a boolean indicating\n"
+            "whether or not the procedure takes rest arguments.\n\n"
+            "For a case-lambda procedure, the arity returned is the one\n"
+            "with the lowest minimum number of arguments, and the highest\n"
+            "maximum number of arguments.\n\n"
+            "If it was not possible to determine the arity of the procedure,\n"
+            "@code{#f} is returned.")
+#define FUNC_NAME s_scm_procedure_minimum_arity
+{
+  int req, opt, rest;
+  
+  if (scm_i_procedure_arity (proc, &req, &opt, &rest))
+    return scm_list_3 (scm_from_int (req),
+                       scm_from_int (opt),
+                       scm_from_bool (rest));
+  else
+    return SCM_BOOL_F;
+}
+#undef FUNC_NAME
 
 SCM_DEFINE (scm_procedure_properties, "procedure-properties", 1, 0, 0, 
            (SCM proc),
@@ -81,21 +108,26 @@ SCM_DEFINE (scm_procedure_properties, "procedure-properties", 1, 0, 0,
 #define FUNC_NAME s_scm_procedure_properties
 {
   SCM ret;
-  int req, opt, rest;
   
   SCM_VALIDATE_PROC (1, proc);
 
-  scm_i_pthread_mutex_lock (&props_lock);
-  ret = scm_hashq_ref (props, proc, SCM_EOL);
-  scm_i_pthread_mutex_unlock (&props_lock);
+  scm_i_pthread_mutex_lock (&overrides_lock);
+  ret = scm_hashq_ref (overrides, proc, SCM_BOOL_F);
+  scm_i_pthread_mutex_unlock (&overrides_lock);
 
-  scm_i_procedure_arity (proc, &req, &opt, &rest);
+  if (scm_is_false (ret))
+    {
+      if (SCM_PROGRAM_P (proc))
+        ret = scm_i_program_properties (proc);
+      else
+        ret = SCM_EOL;
+    }
+  
+#if (SCM_ENABLE_DEPRECATED == 1)
+  ret = scm_acons (scm_sym_arity, scm_procedure_minimum_arity (proc), ret);
+#endif
 
-  return scm_acons (scm_sym_arity,
-                    scm_list_3 (scm_from_int (req),
-                                scm_from_int (opt),
-                                scm_from_bool (rest)),
-                    ret);
+  return ret;
 }
 #undef FUNC_NAME
 
@@ -106,9 +138,14 @@ SCM_DEFINE (scm_set_procedure_properties_x, "set-procedure-properties!", 2, 0, 0
 {
   SCM_VALIDATE_PROC (1, proc);
 
-  scm_i_pthread_mutex_lock (&props_lock);
-  scm_hashq_set_x (props, proc, alist);
-  scm_i_pthread_mutex_unlock (&props_lock);
+#if (SCM_ENABLE_DEPRECATED == 1)
+  if (scm_assq (alist, scm_sym_arity))
+    SCM_MISC_ERROR ("arity is a read-only property", SCM_EOL);
+#endif
+
+  scm_i_pthread_mutex_lock (&overrides_lock);
+  scm_hashq_set_x (overrides, proc, alist);
+  scm_i_pthread_mutex_unlock (&overrides_lock);
 
   return SCM_UNSPECIFIED;
 }
@@ -121,25 +158,14 @@ SCM_DEFINE (scm_procedure_property, "procedure-property", 2, 0, 0,
 {
   SCM_VALIDATE_PROC (1, proc);
 
+#if (SCM_ENABLE_DEPRECATED == 1)
   if (scm_is_eq (key, scm_sym_arity))
-    /* avoid a cons in this case */
-    {
-      int req, opt, rest;
-      scm_i_procedure_arity (proc, &req, &opt, &rest);
-      return scm_list_3 (scm_from_int (req),
-                         scm_from_int (opt),
-                         scm_from_bool (rest));
-    }
-  else
-    {
-      SCM ret;
+    scm_c_issue_deprecation_warning
+      ("Accessing a procedure's arity via `procedure-property' is deprecated.\n"
+       "Use `procedure-minimum-arity instead.");
+#endif
 
-      scm_i_pthread_mutex_lock (&props_lock);
-      ret = scm_hashq_ref (props, proc, SCM_EOL);
-      scm_i_pthread_mutex_unlock (&props_lock);
-
-      return scm_assq_ref (ret, key);
-    }
+  return scm_assq_ref (scm_procedure_properties (proc), key);
 }
 #undef FUNC_NAME
 
@@ -149,17 +175,25 @@ SCM_DEFINE (scm_set_procedure_property_x, "set-procedure-property!", 3, 0, 0,
 	    "@var{val}.")
 #define FUNC_NAME s_scm_set_procedure_property_x
 {
+  SCM props;
+
   SCM_VALIDATE_PROC (1, proc);
 
+#if (SCM_ENABLE_DEPRECATED == 1)
   if (scm_is_eq (key, scm_sym_arity))
-    SCM_MISC_ERROR ("arity is a read-only property", SCM_EOL);
+    SCM_MISC_ERROR ("arity is a deprecated read-only property", SCM_EOL);
+#endif
 
-  scm_i_pthread_mutex_lock (&props_lock);
-  scm_hashq_set_x (props, proc,
-                   scm_assq_set_x (scm_hashq_ref (props, proc,
-                                                  SCM_EOL),
-                                   key, val));
-  scm_i_pthread_mutex_unlock (&props_lock);
+  props = scm_procedure_properties (proc);
+
+#if (SCM_ENABLE_DEPRECATED == 1)
+  /* cdr past the consed-on arity. */
+  props = scm_cdr (props);
+#endif
+
+  scm_i_pthread_mutex_lock (&overrides_lock);
+  scm_hashq_set_x (overrides, proc, scm_assq_set_x (props, key, val));
+  scm_i_pthread_mutex_unlock (&overrides_lock);
 
   return SCM_UNSPECIFIED;
 }
@@ -171,7 +205,7 @@ SCM_DEFINE (scm_set_procedure_property_x, "set-procedure-property!", 3, 0, 0,
 void
 scm_init_procprop ()
 {
-  props = scm_make_weak_key_hash_table (SCM_UNDEFINED);
+  overrides = scm_make_weak_key_hash_table (SCM_UNDEFINED);
 #include "libguile/procprop.x"
 }
 
