@@ -224,14 +224,18 @@
 #define RUN_HOOK1(h, x)
 #endif
 
-#define BOOT_HOOK()	RUN_HOOK (SCM_VM_BOOT_HOOK)
-#define HALT_HOOK()	RUN_HOOK (SCM_VM_HALT_HOOK)
-#define NEXT_HOOK()	RUN_HOOK (SCM_VM_NEXT_HOOK)
-#define BREAK_HOOK()	RUN_HOOK (SCM_VM_BREAK_HOOK)
-#define ENTER_HOOK()	RUN_HOOK (SCM_VM_ENTER_HOOK)
-#define APPLY_HOOK()	RUN_HOOK (SCM_VM_APPLY_HOOK)
-#define EXIT_HOOK()	RUN_HOOK (SCM_VM_EXIT_HOOK)
-#define RETURN_HOOK(n)	RUN_HOOK1 (SCM_VM_RETURN_HOOK, SCM_I_MAKINUM (n))
+#define APPLY_HOOK()                            \
+  RUN_HOOK (SCM_VM_APPLY_HOOK)
+#define PUSH_CONTINUATION_HOOK()                \
+  RUN_HOOK (SCM_VM_PUSH_CONTINUATION_HOOK)
+#define POP_CONTINUATION_HOOK(n)                \
+  RUN_HOOK1 (SCM_VM_POP_CONTINUATION_HOOK, SCM_I_MAKINUM (n))
+#define NEXT_HOOK()                             \
+  RUN_HOOK (SCM_VM_NEXT_HOOK)
+#define ABORT_CONTINUATION_HOOK()               \
+  RUN_HOOK (SCM_VM_ABORT_CONTINUATION_HOOK)
+#define RESTORE_CONTINUATION_HOOK()            \
+  RUN_HOOK (SCM_VM_RESTORE_CONTINUATION_HOOK)
 
 #define VM_HANDLE_INTERRUPTS                     \
   SCM_ASYNC_TICK_WITH_CODE (SYNC_REGISTER ())
