@@ -46,6 +46,9 @@
 (eval-when (compile)
   (define-syntax capture-env
     (syntax-rules ()
+      ((_ (exp ...))
+       (let ((env (exp ...)))
+         (capture-env env)))
       ((_ env)
        (if (null? env)
            (current-module)
@@ -55,7 +58,8 @@
                (and (current-module) the-root-module)
                env)))))
 
-  (define-syntax make-closure
+  ;; Fast case for procedures with fixed arities.
+  (define-syntax make-fixed-closure
     (lambda (x)
       (define *max-static-argument-count* 8)
       (define (make-formals n)
@@ -66,22 +70,17 @@
                  (string (integer->char (+ (char->integer #\a) i))))))
              (iota n)))
       (syntax-case x ()
-        ((_ eval nreq rest? body env) (not (identifier? #'env))
+        ((_ eval nreq body env) (not (identifier? #'env))
          #'(let ((e env))
-             (make-closure eval nreq rest? body e)))
-        ((_ eval nreq rest? body env)
+             (make-fixed-closure eval nreq body e)))
+        ((_ eval nreq body env)
          #`(case nreq
              #,@(map (lambda (nreq)
                        (let ((formals (make-formals nreq)))
                          #`((#,nreq)
-                            (if rest?
-                                (lambda (#,@formals . rest)
-                                  (eval body
-                                        (cons* rest #,@(reverse formals)
-                                               env)))
-                                (lambda (#,@formals)
-                                  (eval body
-                                        (cons* #,@(reverse formals) env)))))))
+                            (lambda (#,@formals)
+                              (eval body
+                                    (cons* #,@(reverse formals) env))))))
                      (iota *max-static-argument-count*))
              (else
               #,(let ((formals (make-formals *max-static-argument-count*)))
@@ -91,13 +90,11 @@
                                (args more))
                         (if (zero? nreq)
                             (eval body
-                                  (if rest?
-                                      (cons args new-env)
-                                      (if (not (null? args))
-                                          (scm-error 'wrong-number-of-args
-                                                     "eval" "Wrong number of arguments"
-                                                     '() #f)
-                                          new-env)))
+                                  (if (null? args)
+                                      new-env
+                                      (scm-error 'wrong-number-of-args
+                                                 "eval" "Wrong number of arguments"
+                                                 '() #f)))
                             (if (null? args)
                                 (scm-error 'wrong-number-of-args
                                            "eval" "Wrong number of arguments"
@@ -218,15 +215,136 @@
 
 (define primitive-eval
   (let ()
+    ;; We pre-generate procedures with fixed arities, up to some number of
+    ;; arguments; see make-fixed-closure above.
+
+    ;; A unique marker for unbound keywords.
+    (define unbound-arg (list 'unbound-arg))
+
+    ;; Procedures with rest, optional, or keyword arguments, potentially with
+    ;; multiple arities, as with case-lambda.
+    (define (make-general-closure env body nreq rest? nopt kw inits alt)
+      (define alt-proc
+        (and alt
+             (let* ((body (car alt))
+                    (nreq (cadr alt))
+                    (rest (if (null? (cddr alt)) #f (caddr alt)))
+                    (tail (and (pair? (cddr alt)) (pair? (cdddr alt)) (cdddr alt)))
+                    (nopt (if tail (car tail) 0))
+                    (kw (and tail (cadr tail)))
+                    (inits (if tail (caddr tail) '()))
+                    (alt (and tail (cadddr tail))))
+               (make-general-closure env body nreq rest nopt kw inits alt))))
+      (lambda %args
+        (let lp ((env env)
+                 (nreq* nreq)
+                 (args %args))
+          (if (> nreq* 0)
+              ;; First, bind required arguments.
+              (if (null? args)
+                  (if alt
+                      (apply alt-proc %args)
+                      (scm-error 'wrong-number-of-args
+                                 "eval" "Wrong number of arguments"
+                                 '() #f))
+                  (lp (cons (car args) env)
+                      (1- nreq*)
+                      (cdr args)))
+              ;; Move on to optional arguments.
+              (if (not kw)
+                  ;; Without keywords, bind optionals from arguments.
+                  (let lp ((env env)
+                           (nopt nopt)
+                           (args args)
+                           (inits inits))
+                    (if (zero? nopt)
+                        (if rest?
+                            (eval body (cons args env))
+                            (if (null? args)
+                                (eval body env)
+                                (if alt
+                                    (apply alt-proc %args)
+                                    (scm-error 'wrong-number-of-args
+                                               "eval" "Wrong number of arguments"
+                                               '() #f))))
+                        (if (null? args)
+                            (lp (cons (eval (car inits) env) env)
+                                (1- nopt) args (cdr inits))
+                            (lp (cons (car args) env)
+                                (1- nopt) (cdr args) (cdr inits)))))
+                  ;; With keywords, we stop binding optionals at the first
+                  ;; keyword.
+                  (let lp ((env env)
+                           (nopt* nopt)
+                           (args args)
+                           (inits inits))
+                    (if (> nopt* 0)
+                        (if (or (null? args) (keyword? (car args)))
+                            (lp (cons (eval (car inits) env) env)
+                                (1- nopt*) args (cdr inits))
+                            (lp (cons (car args) env)
+                                (1- nopt*) (cdr args) (cdr inits)))
+                        ;; Finished with optionals.
+                        (let* ((aok (car kw))
+                               (kw (cdr kw))
+                               (kw-base (+ nopt nreq (if rest? 1 0)))
+                               (imax (let lp ((imax (1- kw-base)) (kw kw))
+                                       (if (null? kw)
+                                           imax
+                                           (lp (max (cdar kw) imax)
+                                               (cdr kw)))))
+                               ;; Fill in kwargs  with "undefined" vals.
+                               (env (let lp ((i kw-base)
+                                             ;; Also, here we bind the rest
+                                             ;; arg, if any.
+                                             (env (if rest? (cons args env) env)))
+                                      (if (<= i imax)
+                                          (lp (1+ i) (cons unbound-arg env))
+                                          env))))
+                          ;; Now scan args for keywords.
+                          (let lp ((args args))
+                            (if (and (pair? args) (pair? (cdr args))
+                                     (keyword? (car args)))
+                                (let ((kw-pair (assq (car args) kw))
+                                      (v (cadr args)))
+                                  (if kw-pair
+                                      ;; Found a known keyword; set its value.
+                                      (list-set! env (- imax (cdr kw-pair)) v)
+                                      ;; Unknown keyword.
+                                      (if (not aok)
+                                          (scm-error 'keyword-argument-error
+                                                     "eval" "Unrecognized keyword"
+                                                     '() #f)))
+                                  (lp (cddr args)))
+                                (if (pair? args)
+                                    (if rest?
+                                        ;; Be lenient parsing rest args.
+                                        (lp (cdr args))
+                                        (scm-error 'keyword-argument-error
+                                                   "eval" "Invalid keyword"
+                                                   '() #f))
+                                    ;; Finished parsing keywords. Fill in
+                                    ;; uninitialized kwargs by evalling init
+                                    ;; expressions in their appropriate
+                                    ;; environment.
+                                    (let lp ((i (- imax kw-base))
+                                             (inits inits))
+                                      (if (pair? inits)
+                                          (let ((tail (list-tail env i)))
+                                            (if (eq? (car tail) unbound-arg)
+                                                (set-car! tail
+                                                          (eval (car inits)
+                                                                (cdr tail))))
+                                            (lp (1- i) (cdr inits)))
+                                          ;; Finally, eval the body.
+                                          (eval body env))))))))))))))
+
     ;; The "engine". EXP is a memoized expression.
     (define (eval exp env)
       (memoized-expression-case exp
         (('lexical-ref n)
-         (let lp ((n n) (env env))
-           (if (zero? n)
-               (car env)
-               (lp (1- n) (cdr env)))))
-      
+         (list-ref env n))
+        
         (('call (f nargs . args))
          (let ((proc (eval f env)))
            (call eval proc nargs args env)))
@@ -235,10 +353,10 @@
          (variable-ref
           (if (variable? var-or-sym)
               var-or-sym
-              (let lp ((env env))
-                (if (pair? env)
-                    (lp (cdr env))
-                    (memoize-variable-access! exp (capture-env env)))))))
+              (memoize-variable-access! exp
+                                        (capture-env (if (pair? env)
+                                                         (cdr (last-pair env))
+                                                         env))))))
 
         (('if (test consequent . alternate))
          (if (eval test env)
@@ -255,9 +373,14 @@
                (lp (cdr inits)
                    (cons (eval (car inits) env) new-env)))))
       
-        (('lambda (nreq rest? . body))
-         (make-closure eval nreq rest? body (capture-env env)))
-        
+        (('lambda (body nreq . tail))
+         (if (null? tail)
+             (make-fixed-closure eval nreq body (capture-env env))
+             (if (null? (cdr tail))
+                 (make-general-closure (capture-env env) body nreq (car tail)
+                                       0 #f '() #f)
+                 (apply make-general-closure (capture-env env) body nreq tail))))
+
         (('begin (first . rest))
          (let lp ((first first) (rest rest))
            (if (null? rest)
@@ -268,10 +391,7 @@
       
         (('lexical-set! (n . x))
          (let ((val (eval x env)))
-           (let lp ((n n) (env env))
-             (if (zero? n)
-                 (set-car! env val)
-                 (lp (1- n) (cdr env))))))
+           (list-set! env n val)))
         
         (('call-with-values (producer . consumer))
          (call-with-values (eval producer env)
@@ -293,10 +413,10 @@
          (variable-set!
           (if (variable? var-or-sym)
               var-or-sym
-              (let lp ((env env))
-                (if (pair? env)
-                    (lp (cdr env))
-                    (memoize-variable-access! exp (capture-env env)))))
+              (memoize-variable-access! exp
+                                        (capture-env (if (pair? env)
+                                                         (cdr (last-pair env))
+                                                         env))))
           (eval x env)))
       
         (('dynwind (in exp . out))
@@ -332,8 +452,8 @@
     (lambda (exp)
       "Evaluate @var{exp} in the current module."
       (eval 
-       (memoize-expression ((or (module-transformer (current-module))
-                                (lambda (x) x))
-                            exp))
+       (memoize-expression 
+        (if (macroexpanded? exp)
+            exp
+            ((module-transformer (current-module)) exp)))
        '()))))
-
