@@ -16,7 +16,7 @@
  * 02110-1301 USA
  */
 
-#ifndef HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
